@@ -3,6 +3,7 @@
 import json
 import logging
 import sys
+from os import environ
 
 from collections import namedtuple
 from urllib.parse import urlparse
@@ -12,12 +13,15 @@ from requests.exceptions import ProxyError, SSLError, RetryError, InvalidHeader,
 from urllib3.exceptions import NewConnectionError, MaxRetryError
 
 from cbw_api_toolbox.__routes__ import ROUTE_AGENTS
+from cbw_api_toolbox.__routes__ import ROUTE_ASSETS
+from cbw_api_toolbox.__routes__ import ROUTE_APPLICATIVE_SCANS
 from cbw_api_toolbox.__routes__ import ROUTE_COMPLIANCE_RULES
 from cbw_api_toolbox.__routes__ import ROUTE_COMPLIANCE_ASSETS
 from cbw_api_toolbox.__routes__ import ROUTE_CVE_ANNOUNCEMENTS
+from cbw_api_toolbox.__routes__ import ROUTE_DOCKER_IMAGES
 from cbw_api_toolbox.__routes__ import ROUTE_GROUPS
 from cbw_api_toolbox.__routes__ import ROUTE_HOSTS
-from cbw_api_toolbox.__routes__ import ROUTE_IMPORTER
+from cbw_api_toolbox.__routes__ import ROUTE_AIRGAPPED_SCRIPTS
 from cbw_api_toolbox.__routes__ import ROUTE_NODES
 from cbw_api_toolbox.__routes__ import ROUTE_PING
 from cbw_api_toolbox.__routes__ import ROUTE_REMOTE_ACCESSES
@@ -30,10 +34,27 @@ from cbw_api_toolbox.cbw_auth import CBWAuth
 class CBWApi: # pylint: disable=R0904
     """Class used to communicate with the CBW API"""
 
-    def __init__(self, api_url, api_key, secret_key, verify_ssl=False):
-        self.api_url = api_url
-        self.api_key = api_key
-        self.secret_key = secret_key
+    def __init__(
+        self,
+        api_url=None,
+        api_key=None,
+        secret_key=None,
+        verify_ssl=False,
+    ):
+
+        try:
+            self.api_url = api_url if api_url is not None else environ["CBW_API_URL"]
+            self.api_key = api_key if api_key is not None else environ["CBW_API_KEY"]
+            self.secret_key = (
+                secret_key if secret_key is not None else environ["CBW_SECRET_KEY"]
+            )
+        except KeyError as error:
+            raise TypeError(
+                (
+                    "CBWApi is missing one of its argument. "
+                    f"You can use the environnement variable {error}."
+                )
+            ) from error
 
         self.verify_ssl = verify_ssl
         self.logger = logging.getLogger(self.__class__.__name__)
@@ -66,11 +87,13 @@ class CBWApi: # pylint: disable=R0904
         except (ConnectionError, ProxyError, SSLError, NewConnectionError, RetryError,
                 InvalidHeader, MaxRetryError):
             self.logger.exception("An error occurred when requesting {}".format(route))
+            sys.exit(-1)
 
         except MissingSchema:
             self.logger.error("An error occurred, please check your API_URL.")
             sys.exit(-1)
 
+        return None
     def _get_pages(self, verb, route, params):
         """ Get one or more pages for a method using api v3 pagination """
         response_list = []
@@ -486,27 +509,27 @@ class CBWApi: # pylint: disable=R0904
 
         return self._cbw_parser(response)
 
-    def fetch_importer_scripts(self, params=None):
-        """GET request to /api/v2/cbw_scans/scripts to get a list of all Importer scanning scripts"""
-        response = self._request("GET", [ROUTE_IMPORTER], params)
+    def fetch_airgapped_scripts(self, params=None):
+        """GET request to /api/v2/cbw_scans/scripts to get a list of all air gapped scanning scripts"""
+        response = self._request("GET", [ROUTE_AIRGAPPED_SCRIPTS], params)
         if response.status_code != 200:
             logging.error("Error::{}".format(response.text))
             return None
 
         return self._cbw_parser(response)
 
-    def fetch_importer_script(self, script_id):
-        """GET request to /api/v2/cbw_scans/scripts/{SCRIPT_ID} to get a specific Importer scanning script"""
-        response = self._request("GET", [ROUTE_IMPORTER, script_id])
+    def fetch_airgapped_script(self, script_id):
+        """GET request to /api/v2/cbw_scans/scripts/{SCRIPT_ID} to get a specific air gapped scanning script"""
+        response = self._request("GET", [ROUTE_AIRGAPPED_SCRIPTS, script_id])
         if response.status_code != 200:
-            logging.error("Error::{}".format(response.text))
+            logging.error("Error for script with id {} :: {}".format(script_id, response.text))
             return None
 
         return self._cbw_parser(response)
 
-    def upload_importer_results(self, content):
-        """POST request to /api/v2/cbw_scans/scripts to upload scanning script result"""
-        response = self._request("POST", [ROUTE_IMPORTER], content)
+    def upload_airgapped_results(self, content):
+        """POST request to /api/v2/cbw_scans/scripts to upload air gapped scanning script result"""
+        response = self._request("POST", [ROUTE_AIRGAPPED_SCRIPTS], content)
         if response.status_code != 204:
             logging.error("Error::{}".format(response.text))
             return None
@@ -624,7 +647,7 @@ class CBWApi: # pylint: disable=R0904
             logging.info("Update stored credential with: {}".format(params))
             return self.verif_response(response)
 
-        logging.error("Error: No stored crecential id was specified for update")
+        logging.error("Error: No stored credential id was specified for update")
         return False
 
     def delete_stored_credential(self, stored_credentials_id):
@@ -636,3 +659,110 @@ class CBWApi: # pylint: disable=R0904
 
         logging.error("Error: no stored_credentials_id was specified for deletion")
         return False
+
+    def docker_images(self, params=None):
+        """GET request to /api/v3/assets/docker_images to list all docker images"""
+        response = self._get_pages("GET", [ROUTE_DOCKER_IMAGES], params)
+        return response
+
+    def docker_image(self, docker_image_id):
+        """GET request to /api/v3/assets/docker_images/{id} to retrieve an docker image"""
+        response = self._request("GET", [ROUTE_DOCKER_IMAGES, docker_image_id])
+        if response.status_code != 200:
+            logging.error("Error server id::{}".format(response.text))
+            return None
+        return self._cbw_parser(response)
+
+    def create_docker_image(self, params):
+        """POST request to /api/v3/assets/docker_images to create a docker image"""
+        logging.error(params)
+        response = self._request("POST", [ROUTE_DOCKER_IMAGES], params)
+        if response.status_code != 201:
+            logging.error("Error::{}".format(response.text))
+            return None
+        return self._cbw_parser(response)
+
+    def update_docker_image(self, docker_image_id, params):
+        """PATCH request to /api/v3/assets/docker_images/<id> to update a docker image"""
+        if  docker_image_id:
+            response = self._request("PATCH", [ROUTE_DOCKER_IMAGES, docker_image_id], params)
+            logging.info("Update docker image with: {}".format(params))
+            return self.verif_response(response)
+
+        logging.error("Error: No docker_image_id was specified for update")
+        return False
+
+    def delete_docker_image(self, docker_image_id):
+        """DELETE request to /api/v3/assets/docker_images/<id> to delete a docker image"""
+        if docker_image_id:
+            logging.info("Deleting docker image with ID {}".format(docker_image_id))
+            response = self._request("DELETE", [ROUTE_DOCKER_IMAGES, docker_image_id])
+            return self.verif_response(response)
+
+        logging.error("Error: no docker_image_id was specified for deletion")
+        return False
+
+    def assets(self, params=None):
+        """GET request to /api/v3/assets/servers to get a list of all assets"""
+        response = self._get_pages("GET", [ROUTE_ASSETS], params)
+
+        return response
+
+    def asset(self, asset_id):
+        """GET request to /api/v3/assets/servers/<asset_id> to get a specific asset"""
+        response = self._request("GET", [ROUTE_ASSETS, asset_id])
+        if response.status_code != 200:
+            logging.error("Error::{}".format(response.text))
+            return None
+
+        return self._cbw_parser(response)
+
+    def delete_asset(self, asset_id):
+        """DELETE request to /api/v3/assets/servers/<asset_id> to delete a asset"""
+        response = self._request("DELETE", [ROUTE_ASSETS, asset_id])
+        if response.status_code != 200:
+            logging.error("Error::{}".format(response.text))
+            return None
+
+        return self._cbw_parser(response)
+
+    def applicative_scans(self, params=None):
+        """GET request to /api/v3/assets/applicative_scans to list all applicative scans"""
+        response = self._get_pages("GET", [ROUTE_APPLICATIVE_SCANS], params)
+        return response
+
+    def applicative_scan(self, applicative_scan_id):
+        """GET request to /api/v3/assets/applicative_scans/{id} to retrieve an applicative scan"""
+        response = self._request("GET", [ROUTE_APPLICATIVE_SCANS, applicative_scan_id])
+        if response.status_code != 200:
+            logging.error("Error server id::{}".format(response.text))
+            return None
+        return self._cbw_parser(response)
+
+    def create_applicative_scan(self, params):
+        """POST request to /api/v3/applicative_scans to launch an applicative scan"""
+        response = self._request("POST", [ROUTE_APPLICATIVE_SCANS], params)
+        if response.status_code != 201:
+            logging.error("Error::{}".format(response.text))
+            return None
+        return self._cbw_parser(response)
+
+    def update_applicative_scan(self, applicative_scan_id, params):
+        """PATCH request to /api/v3/applicative_scans/<id> to update an applicative scan"""
+        if  applicative_scan_id:
+            response = self._request("PATCH", [ROUTE_APPLICATIVE_SCANS, applicative_scan_id], params)
+            logging.info("Update applicative scan with: {}".format(params))
+            return self.verif_response(response)
+
+        logging.error("Error: No stored credential id was specified for update")
+        return False
+
+    def delete_applicative_scan(self, applicative_scan_id):
+        """DELETE request to /api/v3/applicative_scans/<id> to delete an applicative scan"""
+        if applicative_scan_id:
+            logging.info("Deleting applicative scan with ID {}".format(applicative_scan_id))
+            response = self._request("DELETE", [ROUTE_APPLICATIVE_SCANS, applicative_scan_id])
+            return self.verif_response(response)
+
+        logging.error("Error: no applicative_scan_id was specified for deletion")
+        return False
